window.Gleam.initSearch([{"doc":"glitr_convert","title":"glitr_convert","content":"# glitr_convert\n\n[![Package Version](https://img.shields.io/hexpm/v/glitr_convert)](https://hex.pm/packages/glitr_convert)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glitr_convert/)\n\n**Encode and decode from and to Gleam types effortlessly !**\n\nDefine a converter once and encode and decode as much as you want.\n\n## Installation\n\n```sh\ngleam add glitr_convert\n```\n\n## Usage\n\n```gleam\nimport gleam/io\nimport gleam/json\nimport glitr/convert as c\nimport glitr/convert/json as glitr_json\n\npub type Person {\n  Person(name: String, age: Int)\n}\n\npub fn main() {\n  let converter =\n    c.object({\n      use name &lt;- c.field(&quot;name&quot;, fn(v: Person) { Ok(v.name) }, c.string())\n      use age &lt;- c.field(&quot;age&quot;, fn(v: Person) { Ok(v.age) }, c.int())\n      c.success(Person(name:, age:))\n    })\n\n  Person(&quot;Anna&quot;, 21)\n  |&gt; glitr_json.json_encode(converter)\n  |&gt; json.to_string\n  |&gt; io.debug\n  // &#39;{&quot;name&quot;: &quot;Anna&quot;, &quot;age&quot;: 21}&#39;\n\n  &quot;{\\&quot;name\\&quot;: \\&quot;Bob\\&quot;, \\&quot;age\\&quot;: 36}&quot;\n  |&gt; json.decode(glitr_json.json_decode(converter))\n  |&gt; io.debug\n  // Ok(Person(&quot;Bob&quot;, 36))\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/glitr_convert&gt;.\n\n## Features\n\n- Javascript and Erlang targets\n- Converters for all basic types (except BitArray)\n- Define converters for List, Dict, Result &amp; Option\n- Build decoders for custom objects and enums\n- Encode and decode to JSON.\n\n## Potential developments\n\n- Add BitArray support\n- Add Tuple support\n- Add Yaml conversion\n\nFeel free to open PRs and issues if you want more features !\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"glitr/convert","title":"Converter","content":"pub opaque type Converter(a)\n A converter is an object with the data necessary to encode and decode a specific Gleam type.  \n You can build converters using the provided constructors.\n\nSynonyms:\nglitr/convert.Converter\nglitr/convert Converter","url":"glitr/convert.html#Converter"},{"doc":"glitr/convert","title":"GlitrType","content":"pub type GlitrType {\n  String\n  Bool\n  Float\n  Int\n  Null\n  List(of: GlitrType)\n  Dict(key: GlitrType, value: GlitrType)\n  Object(fields: List(#(String, GlitrType)))\n  Optional(of: GlitrType)\n  Result(result: GlitrType, error: GlitrType)\n  Enum(variants: List(#(String, GlitrType)))\n}\n This type is used to define the shape of the data.  \n It isn&#39;t meant to be used directly !  \n It is better to use converters that use GlitrTypes internally to decode data.\nString\n\n\nBool\n\n\nFloat\n\n\nInt\n\n\nNull\n\n\nList(of: GlitrType)\n\n\nDict(key: GlitrType, value: GlitrType)\n\n\nObject(fields: List(#(String, GlitrType)))\n\n\nOptional(of: GlitrType)\n\n\nResult(result: GlitrType, error: GlitrType)\n\n\nEnum(variants: List(#(String, GlitrType)))\n\n\nSynonyms:\nglitr/convert.GlitrType\nglitr/convert GlitrType","url":"glitr/convert.html#GlitrType"},{"doc":"glitr/convert","title":"GlitrValue","content":"pub type GlitrValue {\n  StringValue(value: String)\n  BoolValue(value: Bool)\n  FloatValue(value: Float)\n  IntValue(value: Int)\n  NullValue\n  ListValue(value: List(GlitrValue))\n  DictValue(value: dict.Dict(GlitrValue, GlitrValue))\n  ObjectValue(value: List(#(String, GlitrValue)))\n  OptionalValue(value: option.Option(GlitrValue))\n  ResultValue(value: Result(GlitrValue, GlitrValue))\n  EnumValue(variant: String, value: GlitrValue)\n}\n This type is used to represent data values.  \n It is an intermediate type between encoded data and Gleam types.\n It isn&#39;t meant to be used directly !  \nStringValue(value: String)\n\n\nBoolValue(value: Bool)\n\n\nFloatValue(value: Float)\n\n\nIntValue(value: Int)\n\n\nNullValue\n\n\nListValue(value: List(GlitrValue))\n\n\nDictValue(value: dict.Dict(GlitrValue, GlitrValue))\n\n\nObjectValue(value: List(#(String, GlitrValue)))\n\n\nOptionalValue(value: option.Option(GlitrValue))\n\n\nResultValue(value: Result(GlitrValue, GlitrValue))\n\n\nEnumValue(variant: String, value: GlitrValue)\n\n\nSynonyms:\nglitr/convert.GlitrValue\nglitr/convert GlitrValue","url":"glitr/convert.html#GlitrValue"},{"doc":"glitr/convert","title":"PartialConverter","content":"pub opaque type PartialConverter(base)\n Intermediate type to build a converter for an object type\n\nSynonyms:\nglitr/convert.PartialConverter\nglitr/convert PartialConverter","url":"glitr/convert.html#PartialConverter"},{"doc":"glitr/convert","title":"bool","content":"pub fn bool() -&gt; Converter(Bool)\n Basic converter for a Bool value\nSynonyms:\nglitr/convert.bool\nglitr/convert bool","url":"glitr/convert.html#bool"},{"doc":"glitr/convert","title":"decode","content":"pub fn decode(\n  converter: Converter(a),\n) -&gt; fn(GlitrValue) -&gt; Result(a, List(DecodeError))\n Decode a GlitrValue using the provided converter.\nSynonyms:\nglitr/convert.decode\nglitr/convert decode","url":"glitr/convert.html#decode"},{"doc":"glitr/convert","title":"dict","content":"pub fn dict(\n  key: Converter(a),\n  value: Converter(b),\n) -&gt; Converter(Dict(a, b))\n Basic converter for a Dict value.\n \n `key` is a converter for the keys.\n `value` is a converter for the values.\n \n Example:\n \n let converter: Converter(Dict(String, Int)) = dict(string(), int())\n \nSynonyms:\nglitr/convert.dict\nglitr/convert dict","url":"glitr/convert.html#dict"},{"doc":"glitr/convert","title":"encode","content":"pub fn encode(converter: Converter(a)) -&gt; fn(a) -&gt; GlitrValue\n Encode a value into the corresponding GlitrValue using the converter.  \n If the converter isn&#39;t valid, a NullValue is returned.\nSynonyms:\nglitr/convert.encode\nglitr/convert encode","url":"glitr/convert.html#encode"},{"doc":"glitr/convert","title":"enum","content":"pub fn enum(\n  tags: fn(a) -&gt; String,\n  converters: List(#(String, Converter(a))),\n) -&gt; Converter(a)\n Create a converter for an enum type\n \n `tags` is a function that associate a tag to each variant of the enum\n `converters` is a list of converters, each associated with a tag\n \n Example:\n \n type Action {\n   Open(id: String)\n   Close(id: String)\n }\n \n let open_converter = object({\n   use id &lt;- field(&quot;id&quot;, fn(v: Action) {\n     case v {\n       Open(id) -&gt; Ok(id)\n       _ -&gt; Error(Nil)\n     }\n   }, string())\n   success(Open(id:))\n })\n \n let close_converter = object({\n   use id &lt;- field(&quot;id&quot;, fn(v: Action) {\n     case v {\n       Close(id) -&gt; Ok(id)\n       _ -&gt; Error(Nil)\n     }\n   }, string())\n   success(Close(id:))\n })\n \n let action_converter = enum(\n   fn(v) {\n     case v {\n       Open(_) -&gt; &quot;Open&quot;\n       Close(_) -&gt; &quot;Close&quot;\n     }\n   },\n   [\n     #(&quot;Open&quot;, open_converter),\n     #(&quot;Close&quot;, close_converter),\n   ]\n )\n \nSynonyms:\nglitr/convert.enum\nglitr/convert enum","url":"glitr/convert.html#enum"},{"doc":"glitr/convert","title":"field","content":"pub fn field(\n  field_name: String,\n  field_getter: fn(a) -&gt; Result(b, Nil),\n  field_type: Converter(b),\n  next: fn(b) -&gt; PartialConverter(a),\n) -&gt; PartialConverter(a)\n Add a field to a PartialConverter  \n See `object` for its usage details\n \n &#39;field_name&#39; is the field name that will be used in the converted value. It may not be equal to the actual field name.  \n &#39;field_getter&#39; is a function that returns the value of the field from the complete object.  \n &#39;field_type&#39; is a Converter associated to the type of the field.\nSynonyms:\nglitr/convert.field\nglitr/convert field","url":"glitr/convert.html#field"},{"doc":"glitr/convert","title":"float","content":"pub fn float() -&gt; Converter(Float)\n Basic converter for a Float value\nSynonyms:\nglitr/convert.float\nglitr/convert float","url":"glitr/convert.html#float"},{"doc":"glitr/convert","title":"int","content":"pub fn int() -&gt; Converter(Int)\n Basic converter for a Int value\nSynonyms:\nglitr/convert.int\nglitr/convert int","url":"glitr/convert.html#int"},{"doc":"glitr/convert","title":"list","content":"pub fn list(of: Converter(a)) -&gt; Converter(List(a))\n Basic converter for a List value.   \n \n `of` is a converter for the type of the elements.\nSynonyms:\nglitr/convert.list\nglitr/convert list","url":"glitr/convert.html#list"},{"doc":"glitr/convert","title":"map","content":"pub fn map(\n  converter: Converter(a),\n  encode_map: fn(b) -&gt; a,\n  decode_map: fn(a) -&gt; Result(b, List(DecodeError)),\n  default_value: b,\n) -&gt; Converter(b)\n Create a converter by mapping the encode and decode functions from an existing one\n \n Example:\n \n pub type Date {\n   Date(year: Int, month: Int, day: Int)\n }\n \n // We are storing the date as a string for optimized memory storage\n pub fn date_converter() -&gt; Converter(Date) {\n   string()\n   |&gt; map(\n     fn(v: Date) { [v.year, v.month, v.day] |&gt; list.map(int.to_string) |&gt; string.join(&quot;/&quot;) },\n     fn(v: String) { \n       let elems = string.split(v, &quot;/&quot;)\n       case elems {\n         [y, m, d, ..] -&gt; Date(y, m, d)\n         [y, m]  -&gt; Date(y, m, -1)\n         [y] -&gt; Date(y, -1, -1)\n         [] -&gt; Date(-1, -1, -1)\n       }\n     }\n   )\n }\n \nSynonyms:\nglitr/convert.map\nglitr/convert map","url":"glitr/convert.html#map"},{"doc":"glitr/convert","title":"null","content":"pub fn null() -&gt; Converter(Nil)\n Basic converter for a Nil value\nSynonyms:\nglitr/convert.null\nglitr/convert null","url":"glitr/convert.html#null"},{"doc":"glitr/convert","title":"object","content":"pub fn object(converter: PartialConverter(a)) -&gt; Converter(a)\n Create a Converter from a PartialConverter\n \n Example:\n \n type Person {\n   Person(name: String, age: Int)\n }\n \n let convert = object({\n   use name &lt;- field(&quot;name&quot;, fn(v: Person) { Ok(v.name) }, string())\n   use age &lt;- field(&quot;age&quot;, fn(v: Person) { Ok(v.age) }, int())\n   success(Person(name:, age:))\n })\n \nSynonyms:\nglitr/convert.object\nglitr/convert object","url":"glitr/convert.html#object"},{"doc":"glitr/convert","title":"optional","content":"pub fn optional(of: Converter(a)) -&gt; Converter(Option(a))\n Basic converter for a Option value.\n \n `of` is a converter for the optional value.\nSynonyms:\nglitr/convert.optional\nglitr/convert optional","url":"glitr/convert.html#optional"},{"doc":"glitr/convert","title":"result","content":"pub fn result(\n  res: Converter(a),\n  error: Converter(b),\n) -&gt; Converter(Result(a, b))\n Basic converter for a Result value.\n \n `res` is a converter for the Ok value.\n `error` is a converter for the Error value.\nSynonyms:\nglitr/convert.result\nglitr/convert result","url":"glitr/convert.html#result"},{"doc":"glitr/convert","title":"string","content":"pub fn string() -&gt; Converter(String)\n Basic converter for a String value\nSynonyms:\nglitr/convert.string\nglitr/convert string","url":"glitr/convert.html#string"},{"doc":"glitr/convert","title":"success","content":"pub fn success(c: a) -&gt; PartialConverter(a)\n Used to initialize a PartialConverter  \n See `object` for its usage details\nSynonyms:\nglitr/convert.success\nglitr/convert success","url":"glitr/convert.html#success"},{"doc":"glitr/convert","title":"type_def","content":"pub fn type_def(converter: Converter(a)) -&gt; GlitrType\n Return the GlitrType associated with the converter\nSynonyms:\nglitr/convert.type_def\nglitr/convert type_def","url":"glitr/convert.html#type_def"},{"doc":"glitr/convert","title":"glitr/convert","content":"","url":"glitr/convert.html"},{"doc":"glitr/convert/json","title":"decode_value","content":"pub fn decode_value(\n  of: GlitrType,\n) -&gt; fn(Dynamic) -&gt; Result(GlitrValue, List(DecodeError))\n Decode a JSON value using the specified GlitrType as the shape of the data.  \n Returns the corresponding GlitrValue representation.\n This isn&#39;t meant to be used directly !\nSynonyms:\nglitr/convert/json.decode_value\nglitr/convert/json decode_value","url":"glitr/convert/json.html#decode_value"},{"doc":"glitr/convert/json","title":"encode_value","content":"pub fn encode_value(val: GlitrValue) -&gt; Json\n Encode a GlitrValue into its corresponding JSON representation.  \n This is not meant to be used directly !  \n It is better to use converters.\nSynonyms:\nglitr/convert/json.encode_value\nglitr/convert/json encode_value","url":"glitr/convert/json.html#encode_value"},{"doc":"glitr/convert/json","title":"json_decode","content":"pub fn json_decode(\n  converter: Converter(a),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n Decode a Json value using the provided converter.\nSynonyms:\nglitr/convert/json.json_decode\nglitr/convert/json json_decode","url":"glitr/convert/json.html#json_decode"},{"doc":"glitr/convert/json","title":"json_encode","content":"pub fn json_encode(value: a, converter: Converter(a)) -&gt; Json\n Encode a value into the corresponding Json using the converter.  \n If the converter isn&#39;t valid, a NullValue is returned.\nSynonyms:\nglitr/convert/json.json_encode\nglitr/convert/json json_encode","url":"glitr/convert/json.html#json_encode"},{"doc":"glitr/convert/json","title":"glitr/convert/json","content":"","url":"glitr/convert/json.html"},{"doc":"glitr_convert","title":"glitr_convert","content":"","url":"glitr_convert.html"}]);